function [eumat,esmat,emmat,swgt,swgt_us,scr_seas,scr_us,scr,scrcat,scrcat_us,uwgt,uwgt_us,clustr,pcat,pcat_us,nmout,find_nn,ff,pdn_us,pdn,nn_exp,nn_exp_us]=calc_weights_corr(ens,obs,modelnames,usemdl,plfac,pufac,ffwgt)


%% key function for calculating weights

if nargin<7
    %if no variable weights defined, weight all component variables equally
    ffwgt=[1,1,1,1,1,1,1,1,1,1,1,1];
end

%make fieldname vector
ff=fieldnames(obs);
clear a b c a_us b_us c_us
for i=1:numel(ff)
    %% for variable i, make output arrays (out) for models and
    %% (oout) for obs, (owgt) is area/region weight mask, nmout is
    %% the names of the models

    %% calculate for global mask (all seasons)
[out,oout,owgt,nmout]=makearray(ens,obs,modelnames,usemdl,ff{i},'historical',{'r1'},'wgt');
    %% calculate for US only mask (all seasons)

[out,oout,owgtus,nmout]=makearray(ens,obs,modelnames,usemdl,ff{i}, ...
                                  'historical',{'r1'},'wgtus');
%%% calculate for individual seasons
for j=1:4
[out_seas,oout_seas,owgtus_seas,nmout]=makearray(ens,obs,modelnames,usemdl,ff{i},'historical',{'r1'},'wgtus',j);
%concatenate obs & ensemble, weight by mask
pin_seas=[oout_seas,out_seas].*repmat(owgtus_seas(:,1),1, ...
                                      size(out_seas,2)+1);
%remove mean vector

pin_seas=pin_seas-repmat(mean(pin_seas,2),1,size(out_seas,2)+1);
%remove models with bad/missing data
isgood_seas=find(~isnan(mean(pin_seas,2)));
%[a_seas{i,j} b_seas{i,j}
%c_seas{i,j}]=svds(pin_seas(isgood_seas,:),10);
%calculate inter-model distance matrix for the seasonal data
pd_seas{i,j}=squareform(pdist(pin_seas(isgood_seas,:)'));
%normalize inter-model distance matrix
pdn_seas(:,:,i,j)=pd_seas{i,j}./mean(pd_seas{i,j}(:,1));
%First column is distance from obs to each model
scr_seas(:,i,j)=pdn_seas(2:end,1,i,j);
end
%concatenate obs & ensemble, weight by mask (global)
pin=[oout,out].*repmat(owgt(:,1),1,size(out,2)+1);
%concatenate obs & ensemble, weight by mask (US only)
pin_us=[oout,out].*repmat(owgtus(:,1),1,size(out,2)+1);
%

%remove mean vector (global all seasons)
pin=pin-repmat(mean(pin,2),1,size(out,2)+1);
%remove mean vector (US all seasons)
pin_us=pin_us-repmat(mean(pin_us,2),1,size(out,2)+1);
%find models with bad/missing data
isgood=find(~isnan(mean(pin,2)));
isgood_us=find(~isnan(mean(pin_us,2)));
%calculate inter-model distance matrix for the global all season data
pd{i}=squareform(pdist(pin(isgood,:)','euclidean'));
%calculate inter-model distance matrix for the US all season data
pd_us{i}=squareform(pdist(pin_us(isgood,:)','euclidean'));

%keep models with full data
pinvec{i}=pin_us(isgood,:);
%normalize inter-model distance matrix (global, all season
pdn(:,:,i)=pd{i}./mean(pd{i}(:,1));
%normalize inter-model distance matrix (US only, all season
pdn_us(:,:,i)=pd_us{i}./mean(pd_us{i}(:,1));
%First column is distance from obs to each model (global, all season)
scr(:,i)=pdn(2:end,1,i);
%First column is distance from obs to each model (US, all season)
scr_us(:,i)=pdn_us(2:end,1,i);
end

%Initialize combined distance matrix
pcat=pd{1}*0;
pcat_us=pd{1}*0;

%Add normalized inter-model distances for each variable
for i=1:numel(ff)
  pcat=pcat+pdn(:,:,i)*ffwgt(i);
  pcat_us=pcat_us+pdn_us(:,:,i)*ffwgt(i);
end


%First column of combined distance matrix is mutlivariate distance -
%normalize to ensemble mean
%global, all season
scrcat=pcat(2:end,1)./mean(pcat(2:end,1));
%US, all season
scrcat_us=pcat_us(2:end,1)./mean(pcat_us(2:end,1));

%%calc skill weight using multivariate distance and input skill parameter plfac
%%for US
swgt_us=exp(-(scrcat_us./min(scrcat_us)/plfac).^2);
%% and globally
swgt=exp(-(scrcat./min(scrcat)/plfac).^2);

%%normalize skill score
%% US 
swgt_us=swgt_us/mean(swgt_us);
%% and global
swgt=swgt/mean(swgt);


%% uniqueness parameter is defined by an input parameter - the percentile of the
%% inter-model distance matrix (pufac) for which two models are
%% considered close.  nn_exp is the scale
%% parameter for model uniqueness

%global
nn_exp=prctile(pcat(find(pcat(:)>0)),pufac);
%%and US
nn_exp_us=prctile(pcat_us(find(pcat_us(:)>0)),pufac);

%% Calculate uniqueness weight from inter-model distance matrix and
%% uniqueness parameter
%US 
uwgt_us=sum(exp(-(pcat_us(2:end,2:end)/mean(nn_exp_us)).^2)).^-1;

%and global
uwgt=sum(exp(-(pcat(2:end,2:end)/mean(nn_exp)).^2)).^-1;


%% formulate output
%flag models which are less than 1.5 times the scale of uniqueness
%for later analysis
    find_nn=(pcat_us<(1.5*nn_exp_us))-diag(ones(size(pcat_us,1),1));
    %% some plots require cluster analysis for coloring - calculate
    %% this here
    
    %number of clusters
    n=14;
    %make nice colors
	cols=distinguishable_colors(n);
        %and different symbols for sequential models in each cluster
	smms={'o','s','d','*','+','x','.','^','v','>','<','p','h'};
        %calculate multi-dimensional scaling for diagnostic plots
        %(US, all season)
        p10_mds=mdscale(pcat_us(2:end,2:end),13);
        %do cluster analysis
	[a b c]=kmeans(p10_mds,n);
	%make cluster strcuture for plotting
	clustr.a=a;
	clustr.b=b;
	clustr.c=c;
	clustr.n=n;
	clustr.smms=smms;
	clustr.cols=cols;
        %%make weighted output vectors, and calculate sum sqaure
        %%skill scores for weighted ensemble means compared with
        %%observational vector
        for i=1:numel(ff)
            %            pinvec{i}=pin_us(isgood,:);
            %% Observational vector
            omat=pinvec{i}(:,1);
            %% UNiqueness weighted model vector
            mumat=pinvec{i}(:,2:end)*uwgt_us'/numel(uwgt_us);
            %% skill& uniqueness weighted model vector
            msmat=pinvec{i}(:,2:end)*(uwgt_us.*swgt_us')'/ ...
                  numel(uwgt_us);
            % uniqueness weighted mean SSE
            eumat(i)=sum((mumat-omat).^2);
            %% skill & uniqueness weighted mean SSE
            esmat(i)=sum((msmat-omat).^2);
            %% unweighted mean SSE
            emmat(i)=sum((mean(pinvec{i}(:,2:end),2)-omat).^2);

        end
        